/* Java variant of complicated C solution */

class Sums {

  static int[] array;
  final static int length = 100000;

  static void makeArray()  {
    array = new int[length];
    for (int i = 0; i < length; i++) {
      array[i] = 1;
    }
    array[0] = 2; /* results: 2 300002 599996 */
    /* array[0] = 3; => results: 6 300006 600000 */
  }

  static int convert(int v) {
    int i;
    
    /* Arbitrary time-consuming calculation: */
    for (i = 0; i < 1000; i++) {
      v = ((v * v) + v) % 10;
    }

    return v;
  }

  static int deconvert(int v) {
    return v;
    /* for part 2, use instead: return convert(v); */
  }

  static int doSection(int start, int end, int accum) {
    int i;
    
    for (i = start; i < end; i++) {
      accum += convert(array[i]);
      array[i] = deconvert(accum);
    }
    
    return accum;
  }

  /*****************************************/
  /* No-threads case:                      */
  
  static void sequentialSums()
  {
    doSection(0, length, 0);
  }

  /*****************************************/
  /* Two-threads case:                     */

  static int section2Start;
  static int section3Start;

  static int firstTotal;
  static int secondTotal;
  
  static class DoFirstSection implements PoPPThread.Proc {
    public void run() {
      firstTotal = doSection(0, section2Start, 0);
    }
  }

  static class SumSecondSection implements PoPPThread.Proc {
    public void run() {
      secondTotal = doSection(section2Start, section3Start, 0);
    }
  }

  static class DoSecondSection implements PoPPThread.Proc {
    public void run() {
      doSection(section2Start, section3Start, firstTotal);
    }
  }

  static class DoThirdSection implements PoPPThread.Proc {
    public void run() {
      doSection(section3Start, length, firstTotal + secondTotal);
    }
  }

  static void parallelSums() {

    section2Start = length / 3;
    section3Start = section2Start + ((length - section2Start) / 2);

    try {
      new PoPPThread(new DoFirstSection());
      new PoPPThread(new SumSecondSection());
      PoPPThread.joinThreads();
      new PoPPThread(new DoSecondSection());
      new PoPPThread(new DoThirdSection());
      PoPPThread.joinThreads();
    } catch (InterruptedException e) {
    }
  }

  /*****************************************/

  public static void main(String[] argv) {
    makeArray();

    parallelSums();

    System.out.println(array[0] + " " + array[length / 2] + " " + array[length - 1]);
  }
}
