/* Simple version that uses `array' to store temporary results. */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "popp_thread.h"

static int *array;
static int length = 100000;

static void make_array()
{
  int i;

  array = (int *)malloc(length * sizeof(int));
  for (i = 0; i < length; i++) {
    array[i] = 1;
  }
  array[0] = 2; /* results: 2 300002 599996 */
  /* array[0] = 3;  => results: 6 300006 600000 */
}

static int convert(int v)
{
  int i;

  /* Arbitrary time-consuming calculation: */
  for (i = 0; i < 1000; i++) {
    v = ((v * v) + v) % 10;
  }

  return v;
}

static int deconvert(int v)
{
  return convert(v);
}

static int do_section(int start, int end, int accum)
{
  int i;

  for (i = start; i < end; i++) {
    accum += convert(array[i]);
    array[i] = deconvert(accum);
  }

  return accum;
}

/*****************************************/
/* No-threads case:                      */

int sequential_sums()
{
  return do_section(0, length, 0);
}

/*****************************************/
/* Two-threads case:                    */

static int first_total, second_total;

static int convert_section(int start, int end)
{
  int i, v, accum = 0;

  for (i = start; i < end; i++) {
    v = convert(array[i]);
    array[i] = v;
    accum += v;
  }

  return accum;
}

void convert_first_half(int id)
{
  first_total = convert_section(0, length / 2);
}

void convert_second_half(int id)
{
  second_total = convert_section(length / 2, length);
}

static void deconvert_section(int start, int end, int accum)
{
  int i;

  for (i = start; i < end; i++) {
    accum += array[i];
    array[i] = deconvert(accum);
  }
}

void deconvert_first_half(int id)
{
  deconvert_section(0, length / 2, 0);
}

void deconvert_second_half(int id)
{
  deconvert_section(length / 2, length, first_total);
}

int parallel_sums()
{
  thread_create(convert_first_half, 0);
  thread_create(convert_second_half, 0);
  join_threads();
  thread_create(deconvert_first_half, 0);
  thread_create(deconvert_second_half, 0);
  join_threads();

  return first_total + second_total;
}

/*****************************************/

int main(int argc, char **argv) 
{
  make_array();
  
  parallel_sums();

  printf("%d %d %d\n", array[0], array[length / 2], array[length - 1]);

  return 0;
}
